/*
  ISC License

  Copyright (c) 2023, Antonio SJ Musumeci <trapexit@spawn.link>

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted, provided that the above
  copyright notice and this permission notice appear in all copies.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#pragma once

#include "fuse_readdir_base.hpp"

#include "thread_pool.hpp"

// concurrent open, sequential read
namespace FUSE
{
  class ReadDirCOSR final : public FUSE::ReadDirBase
  {
  public:
    ReadDirCOSR(unsigned concurrency,
                unsigned max_queue_depth);
    ~ReadDirCOSR();

    int operator()(const fuse_req_ctx_t   *ctx,
                   fuse_file_info_t const *ffi,
                   fuse_dirents_t         *buf);

  private:
    ThreadPool _tp;
  };
}
